/**
 * Class        ButtonDebounced.cpp
 * Author       2020-04-13 Charles Geiser (https://www.dodeka.ch)
 * 
 * Purpose      Implements a button class which debounces a mechanical
 *              pushbutton and declares the virtual methods: 
 *                  click(),
 *                  doubleClick()
 *                  longClick()
 * 
 * Board        any   
 * 
 * Remarks      The user of this class has to implement the 3 virtual 
 *              methods in his derived class (see LcdMenuControlButton)
 * 
 * References   http://www.technoblogy.com/show?1YHJ   
 */

#include "ButtonDebounced.h"

void ButtonDebounced::setup()
{
  pinMode(_pin, INPUT_PULLUP);
  _btnState = HIGH;
  _clickCount = 0;
  _msFirstClick = 0; 
}

void ButtonDebounced::loop() 
{
  int prevState = _btnState;
  _btnState = digitalRead(_pin);                   // query button, pressed is LOW
  
  if (prevState == HIGH && _btnState == LOW)       // button pressed
  {
    _msButtonDown = millis();                      // remember time
  }
  else if (prevState == LOW && _btnState == HIGH)  // release button ...
  {
    if (millis() - _msButtonDown < _msDebounce)    // ... button bounces
    {
      // Prellen ignorieren
    }
    else if (millis() - _msButtonDown > _msLongClick)    // time greater than 300 ms
    {
      onLongClick();
    }
    else
    {
      _clickCount++;                            // remember time only of 1st clicks
      if (_clickCount == 1) 
        _msFirstClick = millis();
    }
  }
  else                                          
  {                                             // this branch is run through if there is nothing to do in the loop
    if (_clickCount == 1 && millis() - _msFirstClick > _msDoubleClickGap) // time after 1st click has expired
      {
        _msFirstClick = 0;
        _clickCount = 0;
        onClick();
      }
    else if (_clickCount > 1)                   // more than one click occured 
    {
      _msFirstClick = 0;
      _clickCount = 0;
      onDoubleClick(); 
    }     
  }
}

