/**
 * Header       ButtonDebounced.h
 * Author       2020-10-28 Charles Geiser (https://www.dodeka.ch)
 * 
 * Purpose      Declaration of the class ButtonDebounced. The 3 virtual methods
 *                    onClick()
 *                    onLongClick()
 *                    onDoubleClick()
 *              have to be implemented in a derived class by the user
 *              (see class LcdMenuControlButton) 
 *               
 * Constructor
 * arguments    pin   Pin to which the button is connected to (from ground to pin)
 */

#ifndef _BUTTONDEBOUNCED_H_
#define _BUTTONDEBOUNCED_H_
#include <Arduino.h>

class ButtonDebounced
{
  public:
    ButtonDebounced(uint8_t pin) : _pin(pin) 
    {    
    }

    void setup();
    void loop();

  protected:
    // Virtual methods to be implemented in the derived class
    virtual void onClick();
    virtual void onLongClick();
    virtual void onDoubleClick();

  private:
    const uint8_t _pin;
    uint8_t _clickCount;
    int _btnState;
    unsigned long _msDebounce = 50;        // After 50ms the button should have reached a steady state
    unsigned long _msLongClick = 300;      // A Button that is held longer than 300ms is considered as longClick
    unsigned long _msDoubleClickGap = 250; // Two button clicks within 250ms count as a doubleClick
    unsigned long _msButtonDown;
    unsigned long _msFirstClick;  
};
#endif